@@ -1870,6 +1870,7 @@ static int do_process_startack(struct msghdr *msg, char *buf, int len)
 				leavereason = 0;
 				joining_temp_nodeid = 0;
 				node_state = MEMBER;
+				notify_listeners();
 				sm_member_update(cluster_is_quorate);
 			}
 		}
@@ -3141,8 +3142,7 @@ static int elect_master(struct cluster_node **master_node, int disallow_node)
 
 	for (i = 1; i < sizeof_members_array; i++) {
 		if (members_by_nodeid[i] &&
-		    members_by_nodeid[i]->state == NODESTATE_MEMBER &&
-		    i != disallow_node) {
+		    members_by_nodeid[i]->state == NODESTATE_MEMBER) {
 			*master_node = members_by_nodeid[i];
 			P_MEMB("Elected master is %s\n", (*master_node)->name);
 			return (*master_node)->us;
