@@ -18,7 +18,7 @@
 #include "cnxman-socket.h"
 #include "cman_tool.h"
 
-#define OPTION_STRING		("m:n:v:e:2p:c:r:i:N:t:XVwqh?d")
+#define OPTION_STRING		("m:n:v:e:2p:c:r:i:N:t:o:XVwqh?d")
 #define OP_JOIN			1
 #define OP_LEAVE		2
 #define OP_EXPECTED		3
@@ -177,6 +177,8 @@ static void leave(commandline_t *comline)
 
 	if (comline->remove)
 		flags |= CLUSTER_LEAVEFLAG_REMOVED;
+	if (comline->force)
+		flags |= CLUSTER_LEAVEFLAG_FORCE;
 
 	/* If the join count is != 1 then there are other things using
 	   the cluster and we need to be forced */
@@ -188,7 +190,6 @@ static void leave(commandline_t *comline)
 		if (!comline->force) {
 	    		die("Can't leave cluster while there are %d active subsystems\n", result);
 		}
-		flags |= CLUSTER_LEAVEFLAG_FORCE;
 	}
 
 	/* Unlikely this will be needed, but no point in leaving it out */
@@ -378,6 +379,10 @@ static void decode_arguments(int argc, char *argv[], commandline_t *comline)
 			comline->num_interfaces++;
 			break;
 
+		case 'o':
+			comline->override_nodename = strdup(optarg);
+			break;
+
 		case 'm':
 		        i = comline->num_multicasts;
 			if (i >= MAX_INTERFACES)
@@ -432,7 +437,7 @@ static void decode_arguments(int argc, char *argv[], commandline_t *comline)
 			break;
 
 		case 'c':
-			if (strlen(optarg) > MAX_NODE_NAME_LEN)
+			if (strlen(optarg) >= MAX_CLUSTER_NAME_LEN)
 				die("maximum cluster name length is %d",
 				    MAX_CLUSTER_NAME_LEN);
 			strcpy(comline->clustername, optarg);
@@ -552,7 +557,7 @@ static void check_arguments(commandline_t *comline)
 	if (!comline->clustername[0])
 		die("cluster name not set");
 
-	if (!comline->votes)
+	if (!comline->votes_opt)
 		comline->votes = DEFAULT_VOTES;
 
 	if (!comline->port)
@@ -600,8 +605,9 @@ static void check_arguments(commandline_t *comline)
 	if (comline->nodeid < 0 || comline->nodeid > 4096)
 	        die("Node id must be between 1 and 4096");
 
-	if (strlen(comline->clustername) > MAX_CLUSTER_NAME_LEN) {
-	        die("Cluster name must be <= %d characters long",
+	/* Cluster name needs to include trailing NUL */
+	if (strlen(comline->clustername) > MAX_CLUSTER_NAME_LEN-1) {
+	        die("Cluster name must be < %d characters long",
 		    MAX_CLUSTER_NAME_LEN);
 	}
 
