@@ -58,7 +58,7 @@ static int mark_req2ser=0;
 static int insync_req2ser=0;
 
 
-static void *region_state_alloc(unsigned int gfp_mask, void *pool_data){
+static void *region_state_alloc(int gfp_mask, void *pool_data){
 	return kmalloc(sizeof(struct region_state), gfp_mask);
 }
 
@@ -67,29 +67,36 @@ static void region_state_free(void *element, void *pool_data){
 }
 
 #define BYTE_SHIFT 3
+/*
+ *   <region_size> <uuid> [[no]sync] [block_on_error]
+ */
 static int core_ctr(struct dirty_log *log, struct dm_target *ti,
 		    unsigned int argc, char **argv)
 {
 	enum sync sync = DEFAULTSYNC;
+	int failure_response = FR_NONBLOCK;
 
 	struct log_c *lc;
 	sector_t region_size;
 	unsigned int region_count;
 	size_t bitset_size;
+	int uuid = 0;
+	int i;
 
-	if (argc < 1 || argc > 2) {
-		DMWARN("wrong number of arguments to mirror log");
-		return -EINVAL;
-	}
+	/* Already checked argument count */
 
-	if (argc > 1) {
-		if (!strcmp(argv[1], "sync"))
+	for (i = 1; i < argc; i++) {
+		if (!strcmp(argv[i], "sync"))
 			sync = FORCESYNC;
-		else if (!strcmp(argv[1], "nosync"))
+		else if (!strcmp(argv[i], "nosync"))
 			sync = NOSYNC;
+		else if (!strcmp(argv[i], "block_on_error"))
+			failure_response = FR_BLOCK;
+		else if (!uuid)
+			uuid = i;
 		else {
-			DMWARN("unrecognised sync argument to mirror log: %s",
-			       argv[1]);
+			DMWARN("unrecognised argument to clustered mirror log: %s",
+			       argv[i]);
 			return -EINVAL;
 		}
 	}
@@ -112,6 +119,8 @@ static int core_ctr(struct dirty_log *log, struct dm_target *ti,
 	lc->region_size = region_size;
 	lc->region_count = region_count;
 	lc->sync = sync;
+	lc->failure_response = failure_response;
+	strncpy(lc->uuid, argv[uuid], MAX_NAME_LEN);
 
 	/*
 	 * Work out how many words we need to hold the bitset.
@@ -165,7 +174,7 @@ static void core_dtr(struct dirty_log *log)
 /*----------------------------------------------------------------
  * disk log constructor/destructor
  *
- * argv contains log_device region_size followed optionally by [no]sync
+ *   <device> <region_size> <uuid> [[no]sync] [block_on_error]
  *--------------------------------------------------------------*/
 static int disk_ctr(struct dirty_log *log, struct dm_target *ti,
 		    unsigned int argc, char **argv)
@@ -175,10 +184,7 @@ static int disk_ctr(struct dirty_log *log, struct dm_target *ti,
 	struct log_c *lc;
 	struct dm_dev *dev;
 
-	if (argc < 2 || argc > 3) {
-		DMWARN("wrong number of arguments to disk mirror log");
-		return -EINVAL;
-	}
+	/* already checked argument count */
 
 	r = dm_get_device(ti, argv[0], 0, 0 /* FIXME */,
 			  FMODE_READ | FMODE_WRITE, &dev);
@@ -186,6 +192,7 @@ static int disk_ctr(struct dirty_log *log, struct dm_target *ti,
 		DMWARN("Unable to get device %s", argv[0]);
 		return r;
 	}
+
 	r = core_ctr(log, ti, argc - 1, argv + 1);
 	if (r) {
 		dm_put_device(ti, dev);
@@ -217,11 +224,7 @@ static int disk_ctr(struct dirty_log *log, struct dm_target *ti,
 	size = dm_round_up(lc->bitset_uint32_count * sizeof(uint32_t),
 			   1 << SECTOR_SHIFT);
 	lc->bits_location.count = size >> SECTOR_SHIFT;
-	lc->disk_bits = vmalloc(size);
-	if (!lc->disk_bits) {
-		vfree(lc->disk_header);
-		goto bad;
-	}
+
 	return 0;
 
  bad:
@@ -235,7 +238,6 @@ static void disk_dtr(struct dirty_log *log)
 	struct log_c *lc = (struct log_c *) log->context;
 	dm_put_device(lc->ti, lc->log_dev);
 	vfree(lc->disk_header);
-	vfree(lc->disk_bits);
 	core_dtr(log);
 }
 
@@ -249,7 +251,6 @@ static int run_election(struct log_c *lc){
 	mm_segment_t fs;
 	struct log_request lr;  /* ATTENTION -- could be too much on the stack */
   
-	DMERR("REMOVE ME: running election");
 	memset(&lr, 0, sizeof(lr));
 
 	lr.lr_type = LRT_ELECTION;
@@ -299,8 +300,6 @@ static int run_election(struct log_c *lc){
   
 	if(len > 0){
 		lc->server_id = lr.u.lr_coordinator;
-		DMINFO("New cluster log server (nodeid = %u) for log %s",
-		       lc->server_id, lc->uuid);
 	} else {
 		/* ATTENTION -- what do we do with this ? */
 		DMWARN("Failed to receive election results from server");
@@ -419,7 +418,7 @@ static int _consult_server(struct log_c *lc, region_t region,
 
 	if(len <= 0){
 		/* ATTENTION -- what do we do with this ? */
-//		DMWARN("Failed to recvmsg from cluster log server");
+//		DMWARN("Failed to recvmsg from clustered log server");
 		error = len;
 		*retry = 1;
 		goto fail;
@@ -454,7 +453,7 @@ static int _consult_server(struct log_c *lc, region_t region,
 	if(*retry){
 		request_retry_count++;
 		if(!(request_retry_count & 0x1F)){
-			DMINFO("Cluster mirror retried requests :: %u of %u (%u%%)",
+			DMINFO("Clustered mirror retried requests :: %u of %u (%u%%)",
 			       request_retry_count,
 			       request_count,
 			       dm_div_up(request_retry_count*100, request_count));
@@ -587,97 +586,20 @@ static int consult_server(struct log_c *lc, region_t region,
 	return rtn;
 }
 
-static int mirror_set_count = 0;
+
 static int cluster_connect(void);
 static int cluster_disconnect(void);
-extern int read_header(struct log_c *log);
-extern int write_header(struct log_c *log);
 
-static int record_new_uuid(struct log_c *lc)
-{
-	int r = 0;
-
-	snprintf(lc->header.uuid, MAX_NAME_LEN, "brassow-%lu", jiffies);
-
-	DMERR("REMOVE ME:  writing log uuid: %s", lc->header.uuid);
-	if ((r = write_header(lc)))
-		return r;
-
- reread:
-	ssleep(1);
-	memcpy(lc->uuid, lc->header.uuid, MAX_NAME_LEN);
-
-	if ((r = read_header(lc)))
-		return r;
-
-	if (!strncmp(lc->uuid, lc->header.uuid, MAX_NAME_LEN))
-		return 0;
-
-	DMERR("REMOVE ME:  Log uuid still in flux");
-	DMERR("REMOVE ME:    prev: %s", lc->uuid);
-	DMERR("REMOVE ME:    curr: %s", lc->header.uuid);
-	if (!strncmp(lc->header.uuid, "brassow", 7))
-		goto reread;
-
-	DMERR("REMOVE ME:  Log uuid is being corrupted actively!");
-	return -EIO;
-}
-
-static int set_uuid(struct log_c *lc, int core)
-{
-	int r = 0;
-
-	if (core) {
-		/* WARNING -- UGLY HACK */
-		snprintf(lc->uuid, MAX_NAME_LEN, "none-" SECTOR_FORMAT,
-			 lc->region_count);
-		return 0;
-	}
-
-	if ((r = read_header(lc)))
-		return r;
-
-	if (!strncmp(lc->header.uuid, "brassow", 7)) {
-		while (1) {
-			memcpy(lc->uuid, lc->header.uuid,
-			       MAX_NAME_LEN);
-			DMERR("REMOVE ME1:  log id = %s", lc->uuid);
-			ssleep(1);
-			read_header(lc);
-			if(!strncmp(lc->uuid, lc->header.uuid,
-				    MAX_NAME_LEN))
-				break;
-		}
-	} else 
-		r = record_new_uuid(lc);
-
-	return r;
-}
-
-/*----------------------------------------------------------------
- * cluster log constructor
- *
- * argv contains:
- *   cluster disk <device> <region_size> [sync | nosync]
- *   cluster core <region_size> [sync | nosync]
- *--------------------------------------------------------------*/
 static int cluster_ctr(struct dirty_log *log, struct dm_target *ti,
-		       unsigned int argc, char **argv)
+		       unsigned int argc, char **argv, int disk)
 {
 	int error = 0;
 	struct log_c *lc;
 	struct sockaddr_in saddr_in;
-	int core = 0;
 
-	if(argc < 2){
-		DMERR("Too few arguments to cluster mirror");
-		return -EINVAL;
-	}
-
-	if(!strcmp(argv[0], "core")){
-		core = 1;
-		if ((error = core_ctr(log, ti, (argc - 1), (argv + 1)))) {
-			DMWARN("Cluster mirror:: core_ctr failed");
+	if (!disk) {
+		if ((error = core_ctr(log, ti, argc, argv))) {
+			DMWARN("Clustered mirror:: core_ctr failed");
 			return error;
 		}
 	} else {
@@ -685,15 +607,20 @@ static int cluster_ctr(struct dirty_log *log, struct dm_target *ti,
 		** not actually read the disk.  I suppose, however, that if **
 		** it does in the future, we will simply reread it when a   **
 		** server is started here.................................. */
-		if((error = disk_ctr(log, ti, (argc - 1), (argv + 1)))) {
-			DMWARN("Cluster mirror:: disk_ctr failed");
+
+		if((error = disk_ctr(log, ti, argc, argv))) {
+			DMWARN("Clustered mirror:: disk_ctr failed");
 			return error;
 		}
 	}
 
 	lc = log->context;
 
-	lc->paranoid = 0;
+	if (lc->failure_response != FR_BLOCK) {
+		DMERR("Clustered mirror requires \"block_on_error\" parameter");
+		error = -EINVAL;
+		goto fail;
+	}
 
 	atomic_set(&lc->in_sync, -1);
 
@@ -712,7 +639,7 @@ static int cluster_ctr(struct dirty_log *log, struct dm_target *ti,
 			    &lc->client_sock);
 
 	if(error){
-		DMWARN("unable to create cluster log client socket");
+		DMWARN("unable to create clustered log client socket");
 		goto fail;
 	}
 
@@ -732,7 +659,7 @@ static int cluster_ctr(struct dirty_log *log, struct dm_target *ti,
 	}
 
 	if(error){
-		DMWARN("unable to bind cluster log client socket");
+		DMWARN("unable to bind clustered log client socket");
 		sock_release(lc->client_sock);
 		goto fail;
 	}
@@ -740,8 +667,7 @@ static int cluster_ctr(struct dirty_log *log, struct dm_target *ti,
 	return 0;
 
  fail:
-	/* ATTENTION -- if core is an option we must do appropriate */
-	if (lc->disk_bits)
+	if (lc->log_dev)
 		disk_dtr(log);
 	else
 		core_dtr(log);
@@ -749,6 +675,46 @@ static int cluster_ctr(struct dirty_log *log, struct dm_target *ti,
 	return error;
 }
 
+/*------------------------------------------------------------------
+ * clustered_core log constructor
+ * (preceding args::  <start> <len> mirror clustered_core <log_args>
+ *
+ * Right now, 3 <= argc <= 4.  "block_on_error" is required.
+ *
+ * argv contains:
+ *   <region_size> <uuid> [[no]sync] [block_on_error]
+ *--------------------------------------------------------------*/
+static int cluster_core_ctr(struct dirty_log *log, struct dm_target *ti,
+		       unsigned int argc, char **argv) {
+	if ((argc < 3) || (argc > 4)) {
+		DMERR("Too %s arguments to clustered_core mirror log type.",
+		      (argc < 3) ? "few" : "many");
+		return -EINVAL;
+	}
+
+	return cluster_ctr(log, ti, argc, argv, 0);
+}
+
+/*------------------------------------------------------------------
+ * clustered_disk log constructor
+ * (preceding args::  <start> <len> mirror clustered_core <log_args>
+ *
+ * Right now, 4 <= argc <= 5.  "block_on_error" is required.
+ *
+ * argv contains:
+ *   <disk> <region_size> <uuid> [[no]sync] [block_on_error]
+ *--------------------------------------------------------------*/
+static int cluster_disk_ctr(struct dirty_log *log, struct dm_target *ti,
+		       unsigned int argc, char **argv) {
+	if ((argc < 4) || (argc > 5)) {
+		DMERR("Too %s arguments to clustered_disk mirror log type.",
+		      (argc < 4) ? "few" : "many");
+		return -EINVAL;
+	}
+
+	return cluster_ctr(log, ti, argc, argv, 1);
+}
+
 static void cluster_dtr(struct dirty_log *log)
 {
 	struct log_c *lc = (struct log_c *) log->context;
@@ -760,7 +726,8 @@ static void cluster_dtr(struct dirty_log *log)
 	if(lc->server_id == my_id)
 		consult_server(lc, 0, LRT_MASTER_LEAVING, NULL);
 	sock_release(lc->client_sock);
-	if (lc->disk_bits) 
+
+	if (lc->log_dev) 
 		disk_dtr(log);
 	else
 		core_dtr(log);
@@ -776,7 +743,7 @@ static int cluster_presuspend(struct dirty_log *log)
 
 	atomic_set(&lc->suspended, 1);
 	
-	if (lc->disk_bits && lc->log_dev_failed)
+	if (lc->log_dev && lc->log_dev_failed)
 		complete(&lc->failure_completion);
 
 	return 0;
@@ -816,9 +783,6 @@ static int cluster_postsuspend(struct dirty_log *log){
 static int cluster_resume(struct dirty_log *log){
 	struct log_c *lc = (struct log_c *) log->context;
 
-	DMERR("REMOVE ME: Setting uuid.");
-	set_uuid(lc, (lc->disk_bits)? 0: 1);
-	DMERR("REMOVE ME: uuid is set");
 	resume_server_requests();
 	atomic_set(&lc->suspended, 0);
 
@@ -876,15 +840,8 @@ static int cluster_in_sync(struct dirty_log *log, region_t region, int block)
 
 static int cluster_flush(struct dirty_log *log)
 {
-	struct log_c *lc = (struct log_c *) log->context;
-	if(lc->paranoid){
-		/* there should be no pending requests */
-		return 0;
-	} else {
-		/* flush all clear_region requests to server */
-		/* ATTENTION -- not implemented */
-		return 0;
-	}
+	/* FIXME:  flush all clear_region requests to server */
+	return 0;
 }
 
 static void cluster_mark_region(struct dirty_log *log, region_t region)
@@ -1051,15 +1008,8 @@ static region_t cluster_get_sync_count(struct dirty_log *log)
 	}
 
 	if(rtn >= lc->region_count){
-		if(unlikely(atomic_read(&lc->in_sync) < 0)){
-			DMINFO("Initial mirror status:: IN SYNC");
-		}
-		if(!atomic_read(&lc->in_sync)){
-			DMINFO("Mirror status:: IN SYNC");
-		}
 		atomic_set(&lc->in_sync, 1);
 	} else if(unlikely(atomic_read(&lc->in_sync) < 0)){
-		DMINFO("Initial mirror status:: NOT IN SYNC");
 		atomic_set(&lc->in_sync, 0);
 	}
 
@@ -1070,7 +1020,7 @@ static int cluster_status(struct dirty_log *log, status_type_t status,
 			  char *result, unsigned int maxlen)
 {
 	int sz = 0;
-	int arg_count=2;
+	int arg_count=3;
 	struct log_c *lc = (struct log_c *) log->context;
 
 	switch(status){
@@ -1112,38 +1062,51 @@ static int cluster_status(struct dirty_log *log, status_type_t status,
 */
 		if(lc->sync != DEFAULTSYNC)
 			arg_count++;
-		if(lc->paranoid)
-			arg_count++;
 
-		if (lc->disk_bits)
+		if (lc->log_dev)
 			DMEMIT("3 %s %s %c",
 			       log->type->name,                  /* NAME */
 			       lc->log_dev->name,                /* THE LOG DEVICE */
 			       (lc->log_dev_failed)? 'D' : 'A'); /* LOG DEVICE LIVENESS */
 		else
-			DMEMIT("1 cluster");
+			DMEMIT("1 %s", log->type->name);
 
                 break;
 
         case STATUSTYPE_TABLE:
 		if(lc->sync != DEFAULTSYNC)
 			arg_count++;
-		if(lc->paranoid)
+
+		if (lc->log_dev) {
 			arg_count++;
 
-                DMEMIT("%s %u %s%s " SECTOR_FORMAT " ",
-		       log->type->name,                 /* NAME */
-                       arg_count,                       /* # OF ARGS */
-		       (lc->paranoid)? "paranoid ": "", /* paranoid mode */
-		       lc->log_dev->name,               /* THE LOG DEVICE */
-		       lc->region_size);                /* REGION SIZE */
+			DMEMIT("%s %u %s " SECTOR_FORMAT " %s ",
+			       log->type->name,                 /* NAME */
+			       arg_count,                       /* # OF ARGS */
+			       lc->log_dev->name,               /* THE LOG DEVICE */
+			       lc->region_size,                 /* REGION SIZE */
+			       lc->uuid);                       /* UUID */
+		} else {
+			DMEMIT("%s %u " SECTOR_FORMAT " %s ",
+			       log->type->name,                 /* NAME */
+			       arg_count,                       /* # OF ARGS */
+			       lc->region_size,                 /* REGION SIZE */
+			       lc->uuid);                       /* UUID */
+		}
 		if (lc->sync != DEFAULTSYNC)
 			DMEMIT("%ssync ", lc->sync == NOSYNC ? "no" : "");
+		DMEMIT("block_on_error ");
         }
 
 	return sz;
 }
 
+static int cluster_get_failure_response(struct dirty_log *log)
+{
+	struct log_c *lc = (struct log_c *) log->context;
+	return lc->failure_response;
+}
+
 static int clog_stop(void *data){
 	struct log_c *lc;
 
@@ -1212,6 +1175,8 @@ static struct kcl_service_ops clog_ops = {
 	.finish = clog_finish,
 };
 
+static int mirror_set_count = 0; /* used to prevent multiple cluster [dis]connects */
+
 static int cluster_connect(void)
 {
 	int r;
@@ -1219,18 +1184,16 @@ static int cluster_connect(void)
 	if (mirror_set_count++)
 		return 0;
 
-	DMINFO("Cluster mirror log connecting to CMAN");
-
-	r = kcl_register_service("cluster_log", 11, SERVICE_LEVEL_GDLM, &clog_ops,
+	r = kcl_register_service("clustered_log", 13, SERVICE_LEVEL_GDLM, &clog_ops,
 				 1, NULL, &local_id);
 	if (r) {
-		DMWARN("Couldn't register cluster_log service");
+		DMWARN("Couldn't register clustered_log service");
 		return r;
 	}
 
 	r = start_server();
 	if(r){
-		DMWARN("Unable to start cluster log server daemon");
+		DMWARN("Unable to start clustered log server daemon");
 		kcl_unregister_service(local_id);
 		return r;
 	}
@@ -1251,7 +1214,6 @@ static int cluster_disconnect(void)
 	if (--mirror_set_count)
 		return 0;
 
-	DMINFO("Cluster mirror log disconnecting from CMAN");
 	/* By setting 'shutting_down', the server will not be suspended **
 	** when a stop is received */
 	shutting_down = 1;
@@ -1262,10 +1224,10 @@ static int cluster_disconnect(void)
 	return 0;
 }
 
-static struct dirty_log_type _cluster_type = {
-	.name = "cluster",
+static struct dirty_log_type _clustered_core_type = {
+	.name = "clustered_core",
 	.module = THIS_MODULE,
-	.ctr = cluster_ctr,
+	.ctr = cluster_core_ctr,
 	.dtr = cluster_dtr,
 	.presuspend = cluster_presuspend,
 	.postsuspend = cluster_postsuspend,
@@ -1281,6 +1243,29 @@ static struct dirty_log_type _cluster_type = {
 	.complete_resync_work = cluster_complete_resync_work,
 	.get_sync_count = cluster_get_sync_count,
 	.status = cluster_status,
+	.get_failure_response = cluster_get_failure_response,
+};
+
+static struct dirty_log_type _clustered_disk_type = {
+	.name = "clustered_disk",
+	.module = THIS_MODULE,
+	.ctr = cluster_disk_ctr,
+	.dtr = cluster_dtr,
+	.presuspend = cluster_presuspend,
+	.postsuspend = cluster_postsuspend,
+	.resume = cluster_resume,
+	.get_region_size = cluster_get_region_size,
+	.is_clean = cluster_is_clean,
+	.is_remote_recovering = cluster_is_remote_recovering,
+	.in_sync = cluster_in_sync,
+	.flush = cluster_flush,
+	.mark_region = cluster_mark_region,
+	.clear_region = cluster_clear_region,
+	.get_resync_work = cluster_get_resync_work,
+	.complete_resync_work = cluster_complete_resync_work,
+	.get_sync_count = cluster_get_sync_count,
+	.status = cluster_status,
+	.get_failure_response = cluster_get_failure_response,
 };
 
 #define CMIRROR_RELEASE_NAME "0.1.0"
@@ -1288,7 +1273,7 @@ static int __init cluster_dirty_log_init(void)
 {
 	int r = 0;
 
-        printk("dm-cmirror %s (built %s %s) installed\n",
+        DMINFO("dm-cmirror %s (built %s %s) installed\n",
                CMIRROR_RELEASE_NAME, __DATE__, __TIME__);
 
 	INIT_LIST_HEAD(&clear_region_list);
@@ -1304,9 +1289,18 @@ static int __init cluster_dirty_log_init(void)
 
 	init_waitqueue_head(&suspend_client_queue);
 
-	r = dm_register_dirty_log_type(&_cluster_type);
-	if (r)
-		DMWARN("couldn't register cluster dirty log type");
+	r = dm_register_dirty_log_type(&_clustered_core_type);
+	if (r) {
+		DMWARN("couldn't register clustered_core dirty log type");
+		return r;
+	}
+
+	r = dm_register_dirty_log_type(&_clustered_disk_type);
+	if (r) {
+		DMWARN("couldn't register clustered_disk dirty log type");
+		dm_unregister_dirty_log_type(&_clustered_core_type);
+		return r;
+	}
 
 	return r;
 
@@ -1319,13 +1313,14 @@ static void __exit cluster_dirty_log_exit(void)
 		DMERR("this is a fatal error");
 		BUG();
 	}
-	dm_unregister_dirty_log_type(&_cluster_type);
+	dm_unregister_dirty_log_type(&_clustered_core_type);
+	dm_unregister_dirty_log_type(&_clustered_disk_type);
 }
 
 module_init(cluster_dirty_log_init);
 module_exit(cluster_dirty_log_exit);
 
-MODULE_DESCRIPTION(DM_NAME " cluster capable mirror logs (cluster mirroring)");
+MODULE_DESCRIPTION(DM_NAME " cluster capable mirror logs (clustered mirroring)");
 MODULE_AUTHOR("Jonathan Brassow");
 MODULE_LICENSE("GPL");
 
