@@ -48,7 +48,7 @@ static atomic_t _do_requests;
 static int debug_disk_write = 0;
 extern struct list_head log_list_head;
 
-static void *region_user_alloc(unsigned int gfp_mask, void *pool_data){
+static void *region_user_alloc(int gfp_mask, void *pool_data){
 	return kmalloc(sizeof(struct region_user), gfp_mask);
 }
 
@@ -62,20 +62,20 @@ static void region_user_free(void *element, void *pool_data){
  */
 static inline int log_test_bit(uint32_t *bs, unsigned bit)
 {
-	return test_bit(bit, (unsigned long *) bs) ? 1 : 0;
+	return ext2_test_bit(bit, (unsigned long *) bs) ? 1 : 0;
 }
 
 static inline void log_set_bit(struct log_c *l,
 			       uint32_t *bs, unsigned bit)
 {
-	set_bit(bit, (unsigned long *) bs);
+	ext2_set_bit(bit, (unsigned long *) bs);
 	l->touched = 1;
 }
 
 static inline void log_clear_bit(struct log_c *l,
 				 uint32_t *bs, unsigned bit)
 {
-	clear_bit(bit, (unsigned long *) bs);
+	ext2_clear_bit(bit, (unsigned long *) bs);
 	l->touched = 1;
 }
 
@@ -103,7 +103,7 @@ int read_header(struct log_c *log)
 	int r;
 	unsigned long ebits;
 
-	if (!log->disk_bits)
+	if (!log->log_dev)
 		return 0;
 
 	r = dm_io_sync_vm(1, &log->header_location, READ,
@@ -133,7 +133,7 @@ int write_header(struct log_c *log)
 {
 	unsigned long ebits;
 
-	if (!log->disk_bits)
+	if (!log->log_dev)
 		return 0;
 
 	header_to_disk(&log->header, log->disk_header);
@@ -176,17 +176,15 @@ static int read_bits(struct log_c *log)
 	int r;
 	unsigned long ebits;
 
-	if (!log->disk_bits)
+	if (!log->log_dev)
 		return 0;
 
 	r = dm_io_sync_vm(1, &log->bits_location, READ,
-			  log->disk_bits, &ebits);
+			  log->clean_bits, &ebits);
 
 	if (unlikely(r))
 		return r;
 
-	bits_to_core(log->clean_bits, log->disk_bits,
-		     log->bitset_uint32_count);
 	return 0;
 }
 
@@ -194,14 +192,11 @@ static int write_bits(struct log_c *log)
 {
 	unsigned long ebits;
 	
-	if (!log->disk_bits)
+	if (!log->log_dev)
 		return 0;
 
-	bits_to_disk(log->clean_bits, log->disk_bits,
-		     log->bitset_uint32_count);
-	
 	return dm_io_sync_vm(1, &log->bits_location, WRITE,
-			      log->disk_bits, &ebits);
+			      log->clean_bits, &ebits);
 }
 
 static int count_bits32(uint32_t *addr, unsigned size)
@@ -220,9 +215,9 @@ static int _core_get_resync_work(struct log_c *lc, region_t *region)
 		return 0;
 	}
 	do {
-		*region = find_next_zero_bit((unsigned long *) lc->sync_bits,
-					     lc->region_count,
-					     lc->sync_search);
+		*region = ext2_find_next_zero_bit((unsigned long *) lc->sync_bits,
+						  lc->region_count,
+						  lc->sync_search);
 		lc->sync_search = *region + 1;
 
 		if (*region >= lc->region_count)
@@ -589,7 +584,7 @@ static int server_complete_resync_work(struct log_c *lc, struct log_request *lr)
 
 	/* _must_ not be anyone else using */
 	log_set_bit(lc, lc->clean_bits, lr->u.lr_region);
-
+/*
 	info = (uint32_t)(lc->region_count - lc->sync_count);
 
 	if((info < 10001 && !(info%1000)) ||
@@ -599,7 +594,7 @@ static int server_complete_resync_work(struct log_c *lc, struct log_request *lr)
 		DMINFO(SECTOR_FORMAT " out-of-sync regions remaining.",
 		       lc->region_count - lc->sync_count);
 	}
-
+*/
 	ru = find_ru_by_region(lc, lr->u.lr_region);
 	if (!ru) {
 		DMERR("complete_resync_work attempt on unrecorded region.");
