@@ -415,12 +415,18 @@ int dlm_new_lockspace(char *name, int namelen, void **lockspace, int flags)
 
 	down(&dlmstate_lock);
 	error = init_internal();
-	if (error)
+	if (error) {
+		up(&dlmstate_lock);
 		goto out;
+	}
 
 	error = new_lockspace(name, namelen, lockspace, flags);
- out:
 	up(&dlmstate_lock);
+
+	if (error)
+		dlm_release();
+ out:
+
 	return error;
 }
 
@@ -594,6 +600,8 @@ void dlm_emergency_shutdown()
 	struct dlm_ls *ls;
 	struct dlm_ls *tmp;
 
+	printk("WARNING: dlm_emergency_shutdown\n");
+
 	/* Shut lowcomms down to prevent any socket activity */
 	lowcomms_stop_accept();
 
@@ -611,6 +619,8 @@ void dlm_emergency_shutdown()
 	}
 
 	spin_unlock(&lslist_lock);
+
+	printk("WARNING: dlm_emergency_shutdown\n");
 }
 
 struct dlm_recover *allocate_dlm_recover(void)
